<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns:lav="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/application.xsd"
                exclude-result-prefixes="cfg lav">
    <xsl:output method="xml" omit-xml-declaration="yes"/>
    <xsl:variable name="VERSION">16</xsl:variable>

    <!-- remove namespaces -->
    <xsl:template match="lav:view/lav:namespace | lav:view-template/lav:namespace"/>

    <!-- translate pixtl to processors -->
    <xsl:template match="lav:processors/lav:*[not(self::lav:processor |
            self::lav:repeat | self::lav:element-create-as-parent |
            self::lav:element | self::lav:elements | self::lav:element-ignore | self::lav:elements-ignore)]">
        <processor match="{@match}">
            <xsl:attribute name="type">
                <xsl:choose>
                    <xsl:when test="self::lav:aggregate">AppendAggregateProcessor</xsl:when>
                    <xsl:when test="self::lav:insert-parent">InsertParentProcessor</xsl:when>
                    <xsl:when test="self::lav:insert">InsertProcessor</xsl:when>
                    <xsl:when test="self::lav:merge and @count">MultiMergeProcessor</xsl:when>
                    <xsl:when test="self::lav:merge">MergeProcessor</xsl:when>
                    <xsl:when test="self::lav:move">MoveProcessor</xsl:when>
                    <xsl:when test="self::lav:copy">CopyProcessor</xsl:when>
                    <xsl:when test="self::lav:remove">RemoveProcessor</xsl:when>
                    <xsl:when test="self::lav:rename">RenameProcessor</xsl:when>
                    <xsl:when test="self::lav:replace">ReplaceProcessor</xsl:when>
                    <xsl:when test="self::lav:select and lav:group">SelectGroupByProcessor</xsl:when>
                    <xsl:when test="self::lav:select and lav:distinct">SelectDistinctProcessor</xsl:when>
                    <xsl:when test="self::lav:select and not(lav:*)">SelectProcessor</xsl:when>
                    <xsl:otherwise><xsl:message terminate="yes">Processor not supported: <xsl:value-of
                            select="local-name()"/></xsl:message></xsl:otherwise>
                </xsl:choose>
            </xsl:attribute>
            <xsl:if test="ancestor::lav:view/lav:namespace">
                <namespaces>
                    <xsl:for-each select="ancestor::lav:view/lav:namespace">
                        <entry key="{@prefix}"><xsl:value-of select="@uri"/></entry>
                    </xsl:for-each>
                </namespaces>
            </xsl:if>
            <xsl:apply-templates mode="pixtl" select="@* | * | */@*"/>
        </processor>
    </xsl:template>

    <xsl:template mode="pixtl" match="lav:to-following-sibling">
        <parameter name="destination_axis">following-sibling</parameter>
    </xsl:template>
    <xsl:template mode="pixtl" match="lav:to-ancestor">
        <parameter name="destination_axis">ancestor</parameter>
    </xsl:template>
    <xsl:template mode="pixtl" match="lav:group"/>
    <xsl:template mode="pixtl" match="lav:distinct"/>
    <xsl:template mode="pixtl" match="lav:to-last-child"/>

    <xsl:template mode="pixtl" match="lav:*/@match"/>
    <xsl:template mode="pixtl" match="lav:to-last-child/@values">
        <parameter name="node_values" eval="{.}"/>
    </xsl:template>
    <xsl:template mode="pixtl" match="lav:group/@by">
        <parameter name="group_by" eval="{.}"/>
    </xsl:template>

    <xsl:template mode="pixtl" match="@nodes | lav:to-following-sibling/@predicate | lav:to-ancestor/@predicate">
        <parameter>
            <xsl:attribute name="name"><xsl:apply-templates mode="name" select="."/></xsl:attribute>
            <xsl:attribute name="eval"><xsl:value-of select="."/></xsl:attribute>
        </parameter>
    </xsl:template>
    <xsl:template mode="pixtl" match="@*">
        <parameter>
            <xsl:attribute name="name"><xsl:apply-templates mode="name" select="."/></xsl:attribute>
            <xsl:value-of select="."/>
        </parameter>
    </xsl:template>

    <xsl:template mode="name" match="lav:aggregate/@function">function</xsl:template>
    <xsl:template mode="name" match="lav:group/@sort | lav:distinct/@sort">sort_type</xsl:template>
    <xsl:template mode="name" match="lav:group/@descending | lav:distinct/@descending">sort_descending</xsl:template>
    <xsl:template mode="name" match="lav:select/@depth | lav:remove/@depth">depth</xsl:template>
    <xsl:template mode="name" match="lav:select/@single">single_node</xsl:template>
    <xsl:template mode="name" match="lav:merge/@count">count</xsl:template>

    <xsl:template mode="name" match="lav:insert-parent/@name | lav:rename/@name | lav:to-last-child/@name">node_name</xsl:template>
    <xsl:template mode="name" match="lav:insert/@nodes | lav:replace/@nodes">nodes</xsl:template>
    <xsl:template mode="name" match="lav:to-following-sibling/@as | lav:to-ancestor/@as | lav:insert/@as">destination_as</xsl:template>
    <xsl:template mode="name" match="lav:to-following-sibling/@name | lav:to-ancestor/@name">destination_name</xsl:template>
    <xsl:template mode="name" match="lav:to-following-sibling/@predicate | lav:to-ancestor/@predicate">destination_predicate</xsl:template>
    <xsl:template mode="name" match="@*">
        <xsl:message terminate="yes">Unsupported attribute name: <xsl:value-of select="local-name()"/></xsl:message>
    </xsl:template>

    <!-- common -->
    <xsl:template match="/cfg:config/@version">
        <xsl:attribute name="version">
            <xsl:value-of select="$VERSION"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="@* | comment()" priority="-1">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*" priority="-1">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>